@@ -3,8 +3,8 @@
 #include "networkaccess.h"
 #include "videowidget.h"
 #include "minisplitter.h"
-#include "flickcharm.h"
 #include "constants.h"
+#include "downloadmanager.h"
 
 namespace The {
     QMap<QString, QAction*>* globalActions();
@@ -14,10 +14,6 @@ namespace The {
 
 MediaView::MediaView(QWidget *parent) : QWidget(parent) {
 
-#ifdef APP_DEMO
-    tracksPlayed = 0;
-#endif
-
     reallyStopped = false;
 
     QBoxLayout *layout = new QHBoxLayout();
@@ -118,13 +114,12 @@ MediaView::MediaView(QWidget *parent) : QWidget(parent) {
     workaroundTimer->setInterval(3000);
     connect(workaroundTimer, SIGNAL(timeout()), SLOT(timerPlay()));
 
-    // TODO Enable this on touch devices
-    // FlickCharm *flickCharm = new FlickCharm(this);
-    // flickCharm->activateOn(listView);
-
-}
-
-MediaView::~MediaView() {
+#ifdef APP_DEMO
+    demoTimer = new QTimer(this);
+    demoTimer->setSingleShot(true);
+    demoTimer->setInterval(60000);
+    connect(demoTimer, SIGNAL(timeout()), SLOT(demoMessage()));
+#endif
 
 }
 
@@ -150,7 +145,7 @@ void MediaView::search(SearchParams *searchParams) {
     reallyStopped = false;
 
 #ifdef APP_DEMO
-    tracksPlayed = 0;
+    demoTimer->stop();
 #endif
 
     videoAreaWidget->clear();
@@ -278,6 +273,8 @@ void MediaView::activeRowChanged(int row) {
     QMainWindow* mainWindow = dynamic_cast<QMainWindow*>(window());
     if (mainWindow) mainWindow->statusBar()->showMessage(video->title());
 
+    The::globalActions()->value("download")->setEnabled(DownloadManager::instance()->itemForVideo(video) == 0);
+
     // see you in gotStreamUrl...
 
 }
@@ -285,6 +282,13 @@ void MediaView::activeRowChanged(int row) {
 void MediaView::gotStreamUrl(QUrl streamUrl) {
     if (reallyStopped) return;
 
+    Video *video = static_cast<Video *>(sender());
+    if (!video) {
+        qDebug() << "Cannot get sender";
+        return;
+    }
+    video->disconnect(this);
+
     // go!
     qDebug() << "Playing" << streamUrl.toString();
     mediaObject->setCurrentSource(streamUrl);
@@ -301,8 +305,7 @@ void MediaView::gotStreamUrl(QUrl streamUrl) {
     }
 
 #ifdef APP_DEMO
-    if (tracksPlayed > 1) demoExpired();
-    else tracksPlayed++;
+    demoTimer->start();
 #endif
 
 }
@@ -314,7 +317,7 @@ void MediaView::itemActivated(const QModelIndex &index) {
     else listModel->searchMore();
 }
 
-void MediaView::currentSourceChanged(const Phonon::MediaSource source) {
+void MediaView::currentSourceChanged(const Phonon::MediaSource /* source */ ) {
 
 }
 
@@ -453,14 +456,14 @@ void MediaView::saveSplitterState() {
 }
 
 #ifdef APP_DEMO
-void MediaView::demoExpired() {
+void MediaView::demoMessage() {
+    if (mediaObject->state() != Phonon::PlayingState) return;
     mediaObject->pause();
 
     QMessageBox msgBox;
     msgBox.setIconPixmap(QPixmap(":/images/app.png").scaled(64, 64, Qt::KeepAspectRatio, Qt::SmoothTransformation));
-    msgBox.setText(tr("This is just the demo version of %1.").arg(Constants::APP_NAME) + " " +
-                   tr("It allows you to test the application and see if it works for you.")
-                   );
+    msgBox.setText(tr("This is just the demo version of %1.").arg(Constants::APP_NAME));
+    msgBox.setInformativeText(tr("It allows you to test the application and see if it works for you."));
     msgBox.setModal(true);
 
     QPushButton *quitButton = msgBox.addButton(tr("Continue"), QMessageBox::RejectRole);
@@ -473,7 +476,27 @@ void MediaView::demoExpired() {
     } else {
         mediaObject->play();
     }
-
-    tracksPlayed = 1;
 }
 #endif
+
+void MediaView::downloadVideo() {
+    Video* video = listModel->activeVideo();
+    if (!video) return;
+
+    DownloadManager::instance()->addItem(video);
+
+    // TODO animate
+
+    The::globalActions()->value("downloads")->setVisible(true);
+
+    // The::globalActions()->value("download")->setEnabled(DownloadManager::instance()->itemForVideo(video) == 0);
+
+    QMainWindow* mainWindow = dynamic_cast<QMainWindow*>(window());
+    QString message = tr("Downloading %1").arg(video->title());
+    if (mainWindow) mainWindow->statusBar()->showMessage(message);
+}
+
+void MediaView::fullscreen() {
+    videoAreaWidget->setParent(0);
+    videoAreaWidget->showFullScreen();
+}
