@@ -7,22 +7,27 @@
 #include <phonon/videowidget.h>
 #include "View.h"
 #include "ListModel.h"
-#include "thblackbar.h"
+#include "thlibrary/thblackbar.h"
 #include "searchparams.h"
 #include "playlistwidget.h"
 #include "loadingwidget.h"
 #include "videoareawidget.h"
 
+namespace The {
+    QMap<QString, QAction*>* globalActions();
+}
+
 class MediaView : public QWidget, public View {
     Q_OBJECT
 
 public:
     MediaView(QWidget *parent);
-    ~MediaView();
     void initialize();
 
     // View
-    void appear() {}
+    void appear() {
+        listView->setFocus();
+    }
     void disappear();
     QMap<QString, QVariant> metadata() {
         QMap<QString, QVariant> metadata;
@@ -49,6 +54,8 @@ public slots:
     void moveDownSelected();
     void setPlaylistVisible(bool visible=true);
     void saveSplitterState();
+    void downloadVideo();
+    void fullscreen();
 
 private slots:
     // list/model
@@ -68,6 +75,9 @@ private slots:
     void searchMostViewed();
     // timer
     void timerPlay();
+#ifdef APP_DEMO
+    void demoMessage();
+#endif
 
 private:
 
@@ -101,8 +111,7 @@ private:
     Video *skippedVideo;
 
 #ifdef APP_DEMO
-    void demoExpired();
-    int tracksPlayed;
+    QTimer *demoTimer;
 #endif
 
 };
